float4x4 World;
float4x4 View;
float4x4 Projection;

float3 Index;
 
Texture Texture1;

float timeOfDay = 13;

float3 CameraPosition;
float FogNear = 250;
float FogFar = 300;

float4 HorizonColor;
float4 SunColor;		
float4 NightColor;

float4 MorningTint;		
float4 EveningTint;	

sampler Texture1Sampler = sampler_state
{
	Texture = <Texture1>;
	Filter = MIN_MAG_MIP_POINT;
	AddressU = WRAP;
	AddressV = WRAP;
};

struct VS_IN
{
	byte4 pos : POSITION;
	float2 tex : TEXCOORD;
};
 
struct VS_OUTPUT
{
	float4 pos : POSITION;
	float4 col : COLOR;
	float2 tex : TEXCOORD;
	//float dist : TEXCOORD2;
};
 
VS_OUTPUT VS( VS_IN input )
{
	int x = input.pos.x;// input.pos % (0xFF + 1);
	int y = input.pos.y;//(input.pos % (0xFF00 + 1)) / 256;
	int z = input.pos.z;//(input.pos % (0xFF0000 + 1)) / 65536;
	int w = input.pos.w;//(input.pos % (0xFF000000 + 1) ) / 16777216;
		
	float light = ((float)w)/16;

	float4 position = float4(Index.x * 16 + x, y, Index.z * 16 + z, 1);

	VS_OUTPUT output = (VS_OUTPUT)0;
	
	float4 worldPosition = mul(position, World);
	float4 viewPosition = mul(worldPosition, View);

    output.pos = mul(viewPosition, Projection);

	output.tex = input.tex;

	//output.dist = length(CameraPosition - worldPosition);

	float4 sColor = SunColor;

    if(timeOfDay <= 12)
	{
		sColor *= timeOfDay / 12;	
	}
	else
	{
		sColor *= (timeOfDay - 24) / -12;	
	}

	output.col.rgb = (light);// + (input.LocalLight.rgb * 1.5);
	output.col.a = 1;

	return output;
}
 
float4 PS( VS_OUTPUT input ) : SV_Target
{
	float4 texColor1 = tex2D(Texture1Sampler, input.tex);

	return texColor1;

    float fog = 1;//saturate((input.dist - FogNear) / (FogNear-FogFar));    

    float4 color;
	color.rgb  = texColor1.rgb * input.col.rgb;
	color.a = texColor1.a;

    if(color.a == 0)
    {
        clip(-1);
    }

	float4 topColor = SunColor;	 
    float4 FogColor = HorizonColor;    
    float4 nColor = NightColor;

	nColor *= (4 - input.tex.y) * .125f;

	if(timeOfDay <= 12)
	{
		FogColor *= timeOfDay / 12;	
	}
	else
	{
		FogColor *= (timeOfDay - 24) / -12;	
	}

	FogColor += (MorningTint * .05) * ((24 - timeOfDay)/24);
	FogColor += (EveningTint * .05) * (timeOfDay / 24);	
	topColor += nColor;
	FogColor += nColor;

    float4 outputFogColor = lerp(FogColor, topColor, saturate((input.tex.y)/0.9f));

    return lerp(FogColor, color, fog);
}

//technique BlockTechnique
//{
    //pass Pass1
    //{
        //VertexShader = compile vs_3_0 VertexShaderFunction();
        //PixelShader = compile ps_3_0 PixelShaderFunction();
    //}
//}
 
technique BlockTechnique
{
	pass Pass1
	{
		ZEnable = TRUE;
		ZWriteEnable = TRUE;
		AlphaBlendEnable = FALSE;
		VertexShader = compile vs_3_0 VS();
		PixelShader = compile ps_3_0 PS();
	}
}